@@ -873,11 +873,12 @@ class PlanStage(object):
         function_arn = Variable(
             '%s_lambda_arn' % resource.lambda_function.resource_name
         )
-        return [
+        return self._arn_parse_instructions(function_arn) + [
             models.APICall(
                 method_name='add_permission_for_s3_event',
                 params={'bucket': resource.bucket,
-                        'function_arn': function_arn},
+                        'function_arn': function_arn,
+                        'account_id': Variable('account_id')},
             ),
             (models.APICall(
                 method_name='connect_s3_bucket_to_lambda',
